import argparse

from djspoofer import clients, commands
from djspoofer.models import Fingerprint


class Command(commands.ProxyCommand):
    help = 'Test Proxies'

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "--urls",
            required=True,
            nargs='*',
            help="Target URLs for proxies",
        )
        parser.add_argument(
            "--display-output",
            action=argparse.BooleanOptionalAction,
            help="Display Output",
        )

    def handle(self, *args, **kwargs):
        try:
            fp = Fingerprint.objects.random_desktop(browser=kwargs.get('browser'))
            with clients.desktop_client(fingerprint=fp, proxy_enabled=not kwargs['proxy_disabled']) as client:
                for url in kwargs['urls']:
                    r = client.get(url)
                    if kwargs['display_output']:
                        self.stdout.write(r.text)

        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successful GET for "{url}"'))



# Generated by Django 3.2.13 on 2022-05-15 23:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0003_auto_20220506_2242'),
    ]

    operations = [
        migrations.AddField(
            model_name='h2fingerprint',
            name='priority_frames',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from djstarter import utils

from djspoofer import clients, commands
from djspoofer.models import Fingerprint
from djspoofer.remote.h2fingerprint import h2fingerprint_api


class Command(commands.ProxyCommand):
    help = 'Get H2 Fingerprint'

    def handle(self, *args, **kwargs):
        try:
            fp = Fingerprint.objects.random_desktop(browser=kwargs.get('browser'))
            with clients.desktop_client(fingerprint=fp, proxy_enabled=not kwargs['proxy_disabled']) as client:
                r_h2 = h2fingerprint_api.get_h2_fingerprint(client)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(utils.pretty_dict(r_h2))
            self.stdout.write(self.style.MIGRATE_LABEL('Finished getting H2 Fingerprint'))



import collections
import logging
import time
import random

import h2
from h2 import connection
from h2.exceptions import NoAvailableStreamIDError
from h2.settings import Settings, SettingCodes
from hpack import Decoder, Encoder
from hpack.table import HeaderTable
from httpcore._models import Request, Response
from httpcore._sync import http2

from djspoofer import exceptions, utils as sp_utils
from djspoofer.models import H2Fingerprint

logger = logging.getLogger(__name__)


class NewHTTP2Connection(http2.HTTP2Connection):
    H2_FINGERPRINT_HEADER = b'h2-fingerprint-id'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._h2_fingerprint = None

    def _init_h2_fingerprint(self, request: Request):
        for i, (h_key, h_val) in enumerate(request.headers):
            if h_key == self.H2_FINGERPRINT_HEADER:
                self._h2_fingerprint = H2Fingerprint.objects.get_by_oid(str(h_val, 'utf-8'))
                logger.debug(f'Using H2 Fingerprint: {self._h2_fingerprint}')
                return
        raise exceptions.DJSpooferError(f'Header "{self.H2_FINGERPRINT_HEADER}" missing')

    def handle_request(self, request: Request) -> Response:
        self._init_h2_fingerprint(request)
        self._h2_state = NewH2Connection(config=self.CONFIG, h2_fingerprint=self._h2_fingerprint)
        return super().handle_request(request)

    def _send_connection_init(self, request: Request) -> None:
        """
            The HTTP/2 connection requires some initial setup before we can start
            using individual request/response streams on it.
        """

        self._h2_state.initiate_connection()
        self._h2_state.increment_flow_control_window(self._h2_fingerprint.window_update_increment)
        self._add_priority_frames()

        self._write_outgoing_data(request)

    def _send_request_headers(self, request: Request, stream_id: int) -> None:
        end_stream = not http2.has_body_headers(request)

        headers = self._get_psuedo_headers(request, h2_fingerprint=self._h2_fingerprint) + [
            (k.lower(), v)
            for k, v in request.headers
            if k.lower() not in (
                b"host",
                b"transfer-encoding",
                self.H2_FINGERPRINT_HEADER
            )
        ]

        self._h2_state.send_headers(
            stream_id,
            headers,
            end_stream=end_stream,
            priority_exclusive=bool(self._h2_fingerprint.header_priority_exclusive_bit),
            priority_depends_on=self._h2_fingerprint.header_priority_depends_on_id,
            priority_weight=self._h2_fingerprint.header_priority_weight,
        )
        if self._h2_fingerprint.browser in ('Firefox',):
            self._h2_state.increment_flow_control_window(
                self._h2_fingerprint.window_update_increment, stream_id=stream_id
            )
        self._write_outgoing_data(request)

    def _write_outgoing_data(self, request: Request) -> None:
        time.sleep(random.uniform(.06, .1))     # Small delay ensures packets are chunked into separate packets
        # TODO Investigate better way to separate packets
        super()._write_outgoing_data(request)

    def _add_priority_frames(self):
        if priority_frames := self._h2_fingerprint.priority_frames:
            for pf in sp_utils.PriorityFrameParser(priority_frames).frames:
                self._h2_state.prioritize(
                    stream_id=pf.stream_id,
                    exclusive=bool(pf.exclusivity_bit),
                    depends_on=pf.dependent_stream_id,
                    weight=pf.weight
                )

    @staticmethod
    def _get_psuedo_headers(request, h2_fingerprint):
        # In HTTP/2 the ':authority' pseudo-header is used instead of 'Host'.
        # In order to gracefully handle HTTP/1.1 and HTTP/2 we always require
        # HTTP/1.1 style headers, and map them appropriately if we end up on
        # an HTTP/2 connection.
        authority = [v for k, v in request.headers if k.lower() == b"host"][0]

        header_map = {
            'm': (b":method", request.method),
            'a': (b":authority", authority),
            's': (b":scheme", request.url.scheme),
            'p': (b":path", request.url.target),
        }
        return [header_map[k] for k in h2_fingerprint.psuedo_header_order.split(',')]


class NewH2Connection(connection.H2Connection):
    def __init__(self, h2_fingerprint, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._h2_fingerprint = h2_fingerprint
        self.encoder = NewEncoder(self._h2_fingerprint)
        self.decoder = NewDecoder(self._h2_fingerprint)
        self.local_settings = NewSettings(self._h2_fingerprint)

    def get_next_available_stream_id(self):
        """
        Returns an integer suitable for use as the stream ID for the next
        stream created by this endpoint. For server endpoints, this stream ID
        will be even. For client endpoints, this stream ID will be odd. If no
        stream IDs are available, raises :class:`NoAvailableStreamIDError
        <h2.exceptions.NoAvailableStreamIDError>`.

        .. warning:: The return value from this function does not change until
                     the stream ID has actually been used by sending or pushing
                     headers on that stream. For that reason, it should be
                     called as close as possible to the actual use of the
                     stream ID.

        .. versionadded:: 2.0.0

        :raises: :class:`NoAvailableStreamIDError
            <h2.exceptions.NoAvailableStreamIDError>`
        :returns: The next free stream ID this peer can use to initiate a
            stream.
        :rtype: ``int``
        """
        # No streams have been opened yet, so return the lowest allowed stream
        # ID.
        if not self.highest_outbound_stream_id:
            next_stream_id = self._h2_fingerprint.header_priority_stream_id if self.config.client_side else 2
        else:
            next_stream_id = self.highest_outbound_stream_id + 2
        self.config.logger.debug(
            "Next available stream ID %d", next_stream_id
        )
        if next_stream_id > self.HIGHEST_ALLOWED_STREAM_ID:
            raise NoAvailableStreamIDError("Exhausted allowed stream IDs")

        return next_stream_id


class NewSettings(h2.settings.Settings):
    """
        Allows for setting the settings value in any particular order.
        There is no validation of settings since validation throws errors for missing or invalid values
        Use with caution!
    """
    def __init__(self, h2_fingerprint, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.h2_fingerprint = h2_fingerprint
        self._settings = self.get_initial_values()

    def get_initial_values(self):
        h2_fp = self.h2_fingerprint
        initial_values = {
            SettingCodes.HEADER_TABLE_SIZE: h2_fp.header_table_size,                            # 0x01 (Required)
            SettingCodes.ENABLE_PUSH: int(h2_fp.enable_push) if h2_fp.enable_push else None,    # 0x02 (Required)
            SettingCodes.MAX_CONCURRENT_STREAMS: h2_fp.max_concurrent_streams,                  # 0x03 (Optional)
            SettingCodes.INITIAL_WINDOW_SIZE: h2_fp.initial_window_size,                        # 0x04 (Required)
            SettingCodes.MAX_FRAME_SIZE: h2_fp.max_frame_size,                                  # 0x05 (Required)
            SettingCodes.MAX_HEADER_LIST_SIZE: h2_fp.max_header_list_size,                      # 0x06 (Optional)
        }
        return {k: collections.deque([v]) for k, v in initial_values.items() if v is not None}


class NewDecoder(Decoder):
    def __init__(self, h2_fingerprint, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.header_table = NewHeaderTable(h2_fingerprint)
        # self.max_header_list_size = h2_fingerprint.max_header_list_size   # Firefox has no max header list size
        self.max_allowed_table_size = self.header_table.maxsize


class NewEncoder(Encoder):
    def __init__(self, h2_fingerprint):
        super().__init__()
        self.header_table = NewHeaderTable(h2_fingerprint)


class NewHeaderTable(HeaderTable):
    def __init__(self, h2_fingerprint):
        super().__init__()
        self._maxsize = h2_fingerprint.header_table_size



from djstarter import utils

from djspoofer import clients, commands
from djspoofer.models import Fingerprint
from djspoofer.remote.incolumitas import incolumitas_tls_api


class Command(commands.ProxyCommand):
    help = 'Get TLS Fingerprint'

    def handle(self, *args, **kwargs):
        try:
            fp = Fingerprint.objects.random_desktop(browser=kwargs.get('browser'))
            with clients.desktop_client(fingerprint=fp, proxy_enabled=not kwargs['proxy_disabled']) as client:
                r_tls = incolumitas_tls_api.tls_fingerprint(client)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(utils.pretty_dict(r_tls))
            self.stdout.write(self.style.MIGRATE_LABEL('Finished getting TLS Fingerprint'))



import argparse

from django.core.management.base import BaseCommand


class ProxyCommand(BaseCommand):
    def handle(self, *args, **options):
        pass

    help = 'Test Proxies'

    def add_arguments(self, parser):
        parser.add_argument(
            "--proxy-disabled",
            action=argparse.BooleanOptionalAction,
            help="Proxy Disabled",
        )
        parser.add_argument(
            "--browser",
            type=str,
            required=False,
        )



# Generated by Django 3.2.13 on 2022-05-29 00:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0006_auto_20220515_2357'),
    ]

    operations = [
        migrations.AlterField(
            model_name='h2fingerprint',
            name='header_table_size',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]




from unittest import mock

from django.test import TestCase
from httpcore._models import Origin, Request, URL
from httpcore.backends import sync

from djspoofer import utils
from djspoofer.connections import NewHTTP2Connection


class ConnectionTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.h2_fingerprint = utils.h2_hash_to_h2_fingerprint(
            os='Windows',
            browser='Firefox',
            h2_hash='1:65536;4:131072;5:16384|12517377|15:0:13:42|m,p,a,s',
            priority_frames='3:0:0:201,5:0:0:101,7:0:0:1,9:0:7:1,11:0:3:1,13:0:0:241',
            browser_min_major_version=60,
            browser_max_major_version=110,
        )

    @mock.patch.object(sync, 'SyncStream')
    def test_ok(self, mock_sync_stream):
        mock_sync_stream.write.return_value = None

        origin = Origin(
            scheme=b'http',
            host=b'www.example123xyz.com',
            port=80,
        )
        url = URL(
            url='http://www.example123xyz.com'
        )
        req = Request(
            url=url,
            method='GET',
            headers={
                b'host': 'www.example123xyz.com',
                b'h2-fingerprint-id': str(self.h2_fingerprint.oid)
            })
        new_http2_connection = NewHTTP2Connection(
            origin=origin,
            stream=mock_sync_stream(),
            keepalive_expiry=10.0
        )

        new_http2_connection._receive_response = lambda *args, **kwargs: (200, list())
        r = new_http2_connection.handle_request(req)
        self.assertEquals(r.status, 200)



import logging

from django.conf import settings
from djstarter import decorators

from .exceptions import H2Error

logger = logging.getLogger(__name__)


BASE_URL = 'https://www.mediasploit.com'


@decorators.wrap_exceptions(raise_as=H2Error)
def get_h2_fingerprint(client, *args, **kwargs):
    url = f'{BASE_URL}'

    headers = {
        'user-agent': client.user_agent,
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
        'accept-language': 'en-US,en;q=0.5',
        'accept-encoding': 'gzip, deflate, br',
        'upgrade-insecure-requests': '1',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'te': 'trailers',
    }

    r = client.get(url, headers=headers, *args, **kwargs)
    r.raise_for_status()
    return H2FingerprintResponse(r.json())


class H2FingerprintResponse:
    def __init__(self, data):
        self.fingerprint = data['fingerprint']
        self.settings_frame = data['settings_frame']
        self.window_frame = data['window_frame']
        self.header_priority_frame = data['header_priority_frame']
        self.pseudo_headers = data['pseudo_headers']
        self.user_agent = data['user_agent']



from unittest import mock

import httpx
from django.test import TestCase
from httpx import Request, Response, codes

from djspoofer.remote.twocaptcha import twocaptcha_api, exceptions


class BaseTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.mocked_sleep = mock.patch('time.sleep', return_value=None).start()


class GetSolvedCaptchaTests(BaseTestCase):
    """
        Get Captcha ID Tests
    """

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception
        cls.gcaptcha_token = (
            '03AHJ_Vuve5Asa4koK3KSMyUkCq0vUFCR5Im4CwB7PzO3dCxIo11i53epEraq-uBO5mVm2XRikL8iKOWr0aG50sCuej9bXx5'
            'qcviUGSm4iK4NC_Q88flavWhaTXSh0VxoihBwBjXxwXuJZ-WGN5Sy4dtUl2wbpMqAj8Zwup1vyCaQJWFvRjYGWJ_TQBKTXNB'
            '5CCOgncqLetmJ6B6Cos7qoQyaB8ZzBOTGf5KSP6e-K9niYs772f53Oof6aJeSUDNjiKG9gN3FTrdwKwdnAwEYX-F37sI_vLB'
            '1Zs8NQo0PObHYy0b0sf7WSLkzzcIgW9GR0FwcCCm1P8lB-50GQHPEBJUHNnhJyDzwRoRAkVzrf7UkV8wKCdTwrrWqiYDgbrz'
            'URfHc2ESsp020MicJTasSiXmNRgryt-gf50q5BMkiRH7osm4DoUgsjc_XyQiEmQmxl5sqZP7aKsaE-EM00x59XsPzD3m3YI6'
            'SRCFRUevSyumBd7KmXE8VuzIO9lgnnbka4-eZynZa6vbB9cO3QjLH0xSG3-egcplD1uLGh79wC34RF49Ui3eHwua4S9XHpH6'
            'YBe7gXzz6_mv-o-fxrOuphwfrtwvvi2FGfpTexWvxhqWICMFTTjFBCEGEgj7_IFWEKirXW2RTZCVF0Gid7EtIsoEeZkPbrcU'
            'ISGmgtiJkJ_KojuKwImF0G0CsTlxYTOU2sPsd5o1JDt65wGniQR2IZufnPbbK76Yh_KI2DY4cUxMfcb2fAXcFMc9dcpHg6f9'
            'wBXhUtFYTu6pi5LhhGuhpkiGcv6vWYNxMrpWJW_pV7q8mPilwkAP-zw5MJxkgijl2wDMpM-UUQ_k37FVtf-ndbQAIPG7S469'
            'doZMmb5IZYgvcB4ojqCW3Vz6Q'
        )

    @mock.patch.object(twocaptcha_api, 'get_solved_token')
    @mock.patch.object(twocaptcha_api, 'get_captcha_id')
    def test_ok(self, mock_get_captcha_id, mock_get_solved_token):
        mock_get_captcha_id.return_value = '2122988149'
        mock_get_solved_token.return_value = twocaptcha_api.SolvedTokenResponse(
            g_token=self.gcaptcha_token,
            captcha_id='2122988149'
        )

        with httpx.Client() as client:
            r_captcha = twocaptcha_api.get_solved_captcha(
                client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )
            self.assertEquals(r_captcha.g_token, self.gcaptcha_token)
            self.assertEquals(r_captcha.captcha_id, '2122988149')

    @mock.patch.object(twocaptcha_api, 'report_bad_captcha')
    @mock.patch.object(twocaptcha_api, 'get_solved_token')
    @mock.patch.object(twocaptcha_api, 'get_captcha_id')
    def test_exception(self, mock_get_captcha_id, mock_get_solved_token, mock_report_bad_captcha):
        mock_get_captcha_id.return_value = '2122988149'
        mock_get_solved_token.side_effect = exceptions.CaptchaUnsolvable()
        mock_report_bad_captcha.return_value = twocaptcha_api.TwoCaptchaResponse(
            data={
                'status': 1,
                'request': 'OK_REPORT_RECORDED'
            }
        )

        with self.assertRaises(exceptions.CaptchaUnsolvable):
            with httpx.Client() as client:
                r_captcha = twocaptcha_api.get_solved_captcha(
                    client,
                    proxy='http://example.com:1000',
                    site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                    page_url='https://example.com'
                )
                self.assertEquals(r_captcha.g_token, self.gcaptcha_token)
                self.assertEquals(r_captcha.captcha_id, '2122988149')


class GetCaptchaIDTests(BaseTestCase):
    """
        Get Captcha ID Tests
    """

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception

    @mock.patch.object(httpx, 'Client')
    def test_ok(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "2122988149"
            }
        )

        captcha_id = twocaptcha_api.get_captcha_id(
            mock_client,
            proxy='http://example.com:1000',
            site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
            page_url='https://example.com'
        )
        self.assertEquals(captcha_id, '2122988149')

    @mock.patch.object(httpx, 'Client')
    def test_redirect(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.MOVED_PERMANENTLY,
            json={
                "status": 0,
                "request": "2122988149"
            }
        )

        with self.assertRaises(exceptions.TwoCaptchaError):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )

    @mock.patch.object(httpx, 'Client')
    def test_invalid_response(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 0,
                "request": "2122988149"
            }
        )

        with self.assertRaises(exceptions.InvalidResponse):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )

    @mock.patch.object(httpx, 'Client')
    def test_warn_error(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "ERROR_WRONG_CAPTCHA_ID"
            }
        )

        with self.assertRaises(exceptions.TwoCaptchaError):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )

    @mock.patch.object(httpx, 'Client')
    def test_critical_error(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "ERROR_WRONG_USER_KEY"
            }
        )

        with self.assertRaises(exceptions.CriticalError):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )

    @mock.patch.object(httpx, 'Client')
    def test_captcha_unsolvable(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "ERROR_CAPTCHA_UNSOLVABLE"
            }
        )

        with self.assertRaises(exceptions.CaptchaUnsolvable):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )

    @mock.patch.object(httpx, 'Client')
    def test_captcha_not_ready(self, mock_client):
        mock_client.post.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "CAPCHA_NOT_READY"
            }
        )

        with self.assertRaises(exceptions.CaptchaNotReady):
            twocaptcha_api.get_captcha_id(
                mock_client,
                proxy='http://example.com:1000',
                site_key='6Le-wvkSVVABCPBMRTvw0Q4Muexq1bi0DJwx_mJ-',
                page_url='https://example.com'
            )


class GetSolvedTokenTests(BaseTestCase):
    """
        Get Solved Token Tests
    """

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception
        cls.gcaptcha_token = (
            '03AHJ_Vuve5Asa4koK3KSMyUkCq0vUFCR5Im4CwB7PzO3dCxIo11i53epEraq-uBO5mVm2XRikL8iKOWr0aG50sCuej9bXx5'
            'qcviUGSm4iK4NC_Q88flavWhaTXSh0VxoihBwBjXxwXuJZ-WGN5Sy4dtUl2wbpMqAj8Zwup1vyCaQJWFvRjYGWJ_TQBKTXNB'
            '5CCOgncqLetmJ6B6Cos7qoQyaB8ZzBOTGf5KSP6e-K9niYs772f53Oof6aJeSUDNjiKG9gN3FTrdwKwdnAwEYX-F37sI_vLB'
            '1Zs8NQo0PObHYy0b0sf7WSLkzzcIgW9GR0FwcCCm1P8lB-50GQHPEBJUHNnhJyDzwRoRAkVzrf7UkV8wKCdTwrrWqiYDgbrz'
            'URfHc2ESsp020MicJTasSiXmNRgryt-gf50q5BMkiRH7osm4DoUgsjc_XyQiEmQmxl5sqZP7aKsaE-EM00x59XsPzD3m3YI6'
            'SRCFRUevSyumBd7KmXE8VuzIO9lgnnbka4-eZynZa6vbB9cO3QjLH0xSG3-egcplD1uLGh79wC34RF49Ui3eHwua4S9XHpH6'
            'YBe7gXzz6_mv-o-fxrOuphwfrtwvvi2FGfpTexWvxhqWICMFTTjFBCEGEgj7_IFWEKirXW2RTZCVF0Gid7EtIsoEeZkPbrcU'
            'ISGmgtiJkJ_KojuKwImF0G0CsTlxYTOU2sPsd5o1JDt65wGniQR2IZufnPbbK76Yh_KI2DY4cUxMfcb2fAXcFMc9dcpHg6f9'
            'wBXhUtFYTu6pi5LhhGuhpkiGcv6vWYNxMrpWJW_pV7q8mPilwkAP-zw5MJxkgijl2wDMpM-UUQ_k37FVtf-ndbQAIPG7S469'
            'doZMmb5IZYgvcB4ojqCW3Vz6Q'
        )

    @mock.patch.object(httpx, 'Client')
    def test_ok(self, mock_client):
        mock_client.get.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": self.gcaptcha_token
            }
        )

        r_token = twocaptcha_api.get_solved_token(
            mock_client,
            captcha_id='2122988149',
        )
        self.assertEquals(r_token.g_token, self.gcaptcha_token)

    @mock.patch.object(httpx, 'Client')
    def test_error_wrong_captcha_id(self, mock_client):
        mock_client.get.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "ERROR_WRONG_CAPTCHA_ID"
            }
        )

        with self.assertRaises(exceptions.WarnError):
            twocaptcha_api.get_solved_token(
                mock_client,
                captcha_id='2122988149'
            )


class ReportBadCaptchaTests(BaseTestCase):
    """
        Report Bad Captcha Tests
    """

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception

    @mock.patch.object(httpx, 'Client')
    def test_ok(self, mock_client):
        mock_client.get.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "OK_REPORT_RECORDED"
            }
        )

        r_report = twocaptcha_api.report_bad_captcha(
            mock_client,
            captcha_id='2122988149',
        )
        self.assertEquals(r_report.request, 'OK_REPORT_RECORDED')

    @mock.patch.object(httpx, 'Client')
    def test_invalid_response(self, mock_client):
        mock_client.get.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 0,
                "request": "OK_REPORT_RECORDED"
            }
        )

        with self.assertRaises(exceptions.InvalidResponse):
            twocaptcha_api.report_bad_captcha(
                mock_client,
                captcha_id='2122988149',
            )

    @mock.patch.object(httpx, 'Client')
    def test_invalid_report(self, mock_client):
        mock_client.get.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            json={
                "status": 1,
                "request": "bad_response"
            }
        )

        with self.assertRaises(exceptions.TwoCaptchaError):
            twocaptcha_api.report_bad_captcha(
                mock_client,
                captcha_id='2122988149',
            )



# Generated by Django 3.2.13 on 2022-05-15 23:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0004_h2fingerprint_priority_frames'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='h2fingerprint',
            name='priority_depends_on_id',
        ),
        migrations.RemoveField(
            model_name='h2fingerprint',
            name='priority_exclusive',
        ),
        migrations.RemoveField(
            model_name='h2fingerprint',
            name='priority_stream_id',
        ),
        migrations.RemoveField(
            model_name='h2fingerprint',
            name='priority_weight',
        ),
        migrations.AddField(
            model_name='h2fingerprint',
            name='headers_priority_flags',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
    ]



import logging
from urllib.parse import urlparse

from django.conf import settings
from djstarter import decorators

from .exceptions import CaptchaNotReady, CaptchaUnsolvable, CriticalError, WarnError, InvalidResponse, TwoCaptchaError

logger = logging.getLogger(__name__)

API_KEY = settings.TWO_CAPTCHA_API_KEY
BASE_URL = 'http://2captcha.com'

EXC_MAP = {
    'ERROR_WRONG_CAPTCHA_ID': WarnError,
    'MAX_USER_TURN': WarnError,
    'ERROR_NO_SLOT_AVAILABLE': WarnError,
    'ERROR_WRONG_USER_KEY': CriticalError,
    'ERROR_KEY_DOES_NOT_EXIST': CriticalError,
    'ERROR_ZERO_BALANCE': CriticalError,
    'IP_BANNED': CriticalError,
    'ERROR_GOOGLEKEY': CriticalError,
    'ERROR_CAPTCHA_UNSOLVABLE': CaptchaUnsolvable,
    'CAPCHA_NOT_READY':  CaptchaNotReady
}


@decorators.retry(retry_exceptions=CaptchaUnsolvable, tries=3, delay=0, backoff=0)
def get_solved_captcha(client, proxy, site_key, page_url):
    captcha_id = get_captcha_id(client=client, proxy=proxy, site_key=site_key, page_url=page_url)
    try:
        return get_solved_token(session=client, captcha_id=captcha_id)
    except Exception:
        report_bad_captcha(captcha_id)
        raise CaptchaUnsolvable(captcha_id=captcha_id)


def get_captcha_id(client, proxy, site_key, page_url, pingback=None):
    url = f'{BASE_URL}/in.php'
    proxy_type = urlparse(proxy).scheme.upper()

    params = {
        'key': API_KEY,
        'method': 'userrecaptcha',
        'googlekey': site_key,
        'pageurl': page_url,
        'json': '1',
        'proxy': proxy,
        'proxytype': proxy_type,
        'pingback': pingback
    }
    data = {
        'proxy': proxy,
        'proxytype': proxy_type
    }

    r = client.post(url, params=params, data=data, allow_redirects=False)  # Disable redirects to network splash pages
    if not r.status_code == 200:
        raise TwoCaptchaError(f'Non 200 Response. Proxy: {proxy}, Response: {r.text}')

    r_info = TwoCaptchaResponse(r.json())
    captcha_error_check(r_info)
    return r_info.request


@decorators.retry(retry_exceptions=(CaptchaNotReady,), tries=60, delay=5, backoff=1)
def get_solved_token(client, captcha_id):
    url = f'{BASE_URL}/res.php'
    params = {
        'key': API_KEY,
        'action': 'get',
        'id': captcha_id,
        'json': 1,
    }

    r = client.get(url, params=params)

    r_info = TwoCaptchaResponse(r.json())
    captcha_error_check(r_info)
    return SolvedTokenResponse(g_token=r_info.request, captcha_id=captcha_id)


@decorators.retry(retry_exceptions=(TwoCaptchaError,))
def report_bad_captcha(client, captcha_id):
    url = f'{BASE_URL}/res.php'
    params = {
        'key': API_KEY,
        'action': 'reportbad',
        'id': captcha_id,
        'json': '1',
    }

    r = client.get(url, params=params)

    r_report = TwoCaptchaResponse(r.json())
    captcha_error_check(r_report)
    if r_report.bad_captcha_reported:
        logger.info(f'Reported bad captcha id: {captcha_id}')
        return r_report
    else:
        raise TwoCaptchaError(f'Problem while reporting bad captcha: {r.text}')


@decorators.retry(retry_exceptions=(CaptchaNotReady,))
def register_pingback(client, addr):
    url = f'{BASE_URL}/res.php'
    params = {
        'key': API_KEY,
        'action': 'add_pingback',
        'addr': addr,
        'json': '1',
    }

    r = client.get(url, params=params)

    r_report = TwoCaptchaResponse(r.json())
    captcha_error_check(r_report)


def captcha_error_check(r_info):
    if r_info.status != 1:
        raise InvalidResponse(r_info)
    if exc := EXC_MAP.get(r_info.request):
        raise exc()


class TwoCaptchaResponse:
    def __init__(self, data):
        self.status = data['status']
        self.request = data['request']

    @property
    def bad_captcha_reported(self):
        return self.request == 'OK_REPORT_RECORDED'


class SolvedTokenResponse:
    def __init__(self, g_token, captcha_id):
        self.g_token = g_token
        self.captcha_id = captcha_id



from djstarter import exceptions


class TwoCaptchaError(exceptions.ApiError):
    """Indicate generic exception that involve 2Captcha's API."""


class WarnError(TwoCaptchaError):
    """Indicate warn exception that involve 2Captcha's API."""


class CriticalError(TwoCaptchaError):
    """Indicate critical exception that involve 2Captcha's API."""


class CaptchaUnsolvable(TwoCaptchaError):
    """Indicate captcha was unsolvable."""


class CaptchaNotReady(TwoCaptchaError):
    """Captcha is not Ready Yet"""


class InvalidResponse(TwoCaptchaError):
    """Response is not valid"""
